"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllMetadata = exports.getOwnMetadata = exports.getMetadataForApplyTo = exports.getMetadata = exports.mergeMetadata = exports.setMetadata = void 0;
const types_1 = require("./types");
const storage = new Map();
function setMetadata(data, targetClass, memberName, parIndex) {
    var _a;
    const getMetadataOption = (opt) => (Object.assign({ inherit: true, allowMultiple: true, applyTo: [], removeApplied: true }, opt));
    const opt = data[types_1.DecoratorOptionId] = getMetadataOption(data[types_1.DecoratorOptionId]);
    if (!opt.allowMultiple && !data[types_1.DecoratorId]) {
        throw new Error(`Reflect Error: Decorator with allowMultiple set to false must have DecoratorId property in ${targetClass.name}`);
    }
    const applyTo = typeof opt.applyTo === "string" ? [opt.applyTo] : opt.applyTo;
    const meta = (_a = storage.get(targetClass)) !== null && _a !== void 0 ? _a : [];
    if (applyTo.length === 0)
        meta.push({ targetClass, memberName, parIndex, data });
    else {
        for (const apply of applyTo) {
            meta.push({ targetClass, memberName: apply, data });
        }
        meta.push({ targetClass, memberName, parIndex, data });
    }
    storage.set(targetClass, meta);
}
exports.setMetadata = setMetadata;
function getMetadataFromStorage(target, memberName, parIndex) {
    var _a;
    return ((_a = storage.get(target)) !== null && _a !== void 0 ? _a : [])
        .filter(x => x.memberName === memberName && x.parIndex === parIndex)
        .filter(x => {
        const opt = x.data[types_1.DecoratorOptionId];
        return opt.applyTo.length === 0;
    })
        .map(x => x.data);
}
function mergeMetadata(childMeta, parentMeta, useInherit = true) {
    const result = [...childMeta];
    for (const parent of parentMeta) {
        const copyExists = () => !!childMeta.find(x => x[types_1.DecoratorId] !== undefined && x[types_1.DecoratorId] === parent[types_1.DecoratorId]);
        const option = parent[types_1.DecoratorOptionId];
        if (useInherit && !option.inherit)
            continue;
        if (!option.allowMultiple && copyExists())
            continue;
        result.push(parent);
    }
    return result;
}
exports.mergeMetadata = mergeMetadata;
function getMetadata(targetClass, memberName, parIndex) {
    const parent = Object.getPrototypeOf(targetClass);
    const parentMeta = !!parent.prototype ? getMetadata(parent, memberName, parIndex) : [];
    const childMeta = getMetadataFromStorage(targetClass, memberName, parIndex);
    return mergeMetadata(childMeta, parentMeta);
}
exports.getMetadata = getMetadata;
function getMetadataForApplyTo(targetClass, memberName, parIndex) {
    var _a;
    return ((_a = storage.get(targetClass)) !== null && _a !== void 0 ? _a : [])
        .filter(x => x.memberName === memberName && x.parIndex === parIndex)
        .filter(x => {
        const opt = x.data[types_1.DecoratorOptionId];
        return opt.applyTo.length > 0;
    })
        .map(x => x.data);
}
exports.getMetadataForApplyTo = getMetadataForApplyTo;
function getOwnMetadata(targetClass, memberName, parIndex) {
    return getMetadataFromStorage(targetClass, memberName, parIndex);
}
exports.getOwnMetadata = getOwnMetadata;
function getAllMetadata(targetClass) {
    return storage.get(targetClass);
}
exports.getAllMetadata = getAllMetadata;
