import { Class, ClassReflection, ConstructorReflection, MethodReflection, ParameterPropertyReflection, ParameterReflection, PropertyReflection, Reflection } from "./types";
declare type TypedReflection = ClassReflection | MethodReflection | PropertyReflection | ParameterReflection | ConstructorReflection | ParameterPropertyReflection;
declare type WalkMemberVisitor = (value: TypedReflection, ctx: WalkMemberContext) => TypedReflection | undefined;
interface WalkMemberContext {
    target: Class;
    classPath: Class[];
    parent: Reflection;
    memberVisitor: WalkMemberVisitor;
}
interface WalkParentContext {
    target: Class;
    memberVisitor: WalkMemberVisitor;
    classPath: Class[];
}
declare class GenericMap {
    private maps;
    constructor(types: Class[]);
    private createMaps;
    private getTemplates;
    private getTypes;
    get(rawType: string | string[]): Class<any> | Class<any>[];
}
declare namespace memberVisitors {
    function addsDesignTypes(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection;
    function addsDecorators(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection;
    function addsApplyToDecorator(meta: TypedReflection, ctx: WalkMemberContext): ParameterReflection | ConstructorReflection | {
        decorators: any[];
        kind: "Class";
        ctor: ConstructorReflection;
        methods: MethodReflection[];
        properties: PropertyReflection[];
        removedDecorators?: any[] | undefined;
        type: Class<any>;
        super: Class<any>;
        typeClassification?: "Class" | "Array" | "Primitive" | undefined;
        name: string;
    } | {
        decorators: any[];
        index: number;
        isParameter: boolean;
        kind: "Property";
        type?: any;
        get?: any;
        set?: any;
        typeClassification?: "Class" | "Array" | "Primitive" | undefined;
        name: string;
    } | {
        decorators: any[];
        kind: "Property";
        type?: any;
        get?: any;
        set?: any;
        typeClassification?: "Class" | "Array" | "Primitive" | undefined;
        name: string;
    } | {
        decorators: any[];
        kind: "Method";
        parameters: ParameterReflection[];
        returnType: any;
        typeClassification?: "Class" | "Array" | "Primitive" | undefined;
        name: string;
    };
    function removeGenericTypeOverridden(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection;
    function addsTypeOverridden(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection;
    function addsGenericOverridden(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection;
    function addsTypeClassification(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection | undefined;
    function addsParameterProperties(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection | undefined;
    function removeIgnored(meta: TypedReflection, ctx: WalkMemberContext): TypedReflection | undefined;
}
/**
 * Walk into type member metadata (properties, parameters, methods, ctor etc)
 * @param ref type metadata
 * @param ctx traversal context
 */
declare function walkReflectionMembers(ref: TypedReflection, ctx: WalkMemberContext): ClassReflection;
/**
 * Walk into type super class.
 * @param type type to reflect
 */
declare function walkTypeMembersRecursive(type: Class, ctx: WalkParentContext): ClassReflection;
export { walkTypeMembersRecursive, walkReflectionMembers, memberVisitors, WalkMemberVisitor, GenericMap };
